@@ -195,6 +195,12 @@
 #' @param ticks an optional vector of age ticks to be added to the
 #'     concordia line to override \code{IsoplotR}'s default spacing,
 #'     which is based on \code{R}'s \code{pretty} function.
+#' @param hide vector with indices of aliquots that should be removed
+#'     from the concordia diagram
+#' @param omit vector with indices of aliquots that should be plotted
+#'     but omitted from concordia or discordia age calculation
+#' @param omit.col colour that should be used for the omitted
+#'     aliquots.
 #' @param ... optional arguments to the generic \code{plot} function
 #'
 #' @examples
@@ -218,50 +224,42 @@ concordia <- function(x,tlim=NULL,alpha=0.05,wetherill=TRUE,
                       ellipse.col=c("#00FF0080","#FF000080"),
                       concordia.col='darksalmon',exterr=FALSE,
                       show.age=0,sigdig=2,common.Pb=0,ticks=NULL,
-                      anchor=list(FALSE,NA),...){
-    if (common.Pb>0) X <- common.Pb.correction(x,option=common.Pb)
-    else X <- x
-    lims <- prepare.concordia.line(X,tlim=tlim,wetherill=wetherill,...)
+                      anchor=list(FALSE,NA),hide=NULL,omit=NULL,
+                      omit.col=NA,...){
+    ns <- length(x)
+    plotit <- (1:ns)%ni%hide
+    calcit <- (1:ns)%ni%c(hide,omit)
+    if (common.Pb<1) X <- x
+    else X <- common.Pb.correction(x,option=common.Pb)
+    X2plot <- subset(X,subset=plotit)
+    lims <- prepare.concordia.line(X2plot,tlim=tlim,wetherill=wetherill,...)
     fit <- NULL
     if (show.age>1){
-        fit <- concordia.intersection.ludwig(x,wetherill=wetherill,
+        x2calc <- subset(x,subset=calcit)
+        fit <- concordia.intersection.ludwig(x2calc,wetherill=wetherill,
                                              exterr=exterr,alpha=alpha,
                                              model=(show.age-1),anchor=anchor)
         discordia.line(fit,wetherill=wetherill)
-        fit$n <- length(x)
+        fit$n <- length(x2calc)
         graphics::title(discordia.title(fit,wetherill=wetherill,sigdig=sigdig))
     }
-    plot.concordia.line(X,lims=lims,wetherill=wetherill,col=concordia.col,
+    plot.concordia.line(X2plot,lims=lims,wetherill=wetherill,col=concordia.col,
                         alpha=alpha,exterr=exterr,ticks=ticks)
-    ns <- length(x)
-    ellipse.cols <- set.ellipse.colours(ns=ns,levels=levels,col=ellipse.col)
-    for (i in 1:ns){
-        if (wetherill) xyc <- wetherill(X,i)
-        else xyc <- tera.wasserburg(X,i)
-        x0 <- xyc$x[1]
-        y0 <- xyc$x[2]
-        if (show.age==3){
-            pch <- 21
-            pcex <- 1
-        } else {
-            covmat <- xyc$cov
-            ell <- ellipse(x0,y0,covmat,alpha=alpha)
-            graphics::polygon(ell,col=ellipse.cols[i])
-            pch <- 19
-            pcex <- 0.25
-        }
-        if (show.numbers) graphics::text(x0,y0,i)
-        else graphics::points(x0,y0,pch=pch,cex=pcex*graphics::par('cex'))
-    }
+    d <- data2york(X,wetherill=wetherill)
+    scatterplot(d,alpha=alpha,show.numbers=show.numbers,
+                show.ellipses=1*(show.age!=3),levels=levels,
+                clabel=clabel,ellipse.col=ellipse.col,new.plot=FALSE,
+                hide=hide,omit=omit,omit.col=omit.col,addcolourbar=FALSE,...)
     if (show.age==1){
-        fit <- concordia.age(X,wetherill=wetherill,
+        X2calc <- subset(X,subset=calcit)
+        fit <- concordia.age(X2calc,wetherill=wetherill,
                              exterr=exterr,alpha=alpha)
         ell <- ellipse(fit$x[1],fit$x[2],fit$cov)
         graphics::polygon(ell,col='white')
-        fit$n <- length(x)
+        fit$n <- length(X2calc)
         graphics::title(concordia.title(fit,sigdig=sigdig))
     }
-    colourbar(z=levels,col=ellipse.col,clabel=clabel)
+    colourbar(z=levels[calcit],col=ellipse.col,clabel=clabel)
     invisible(fit)
 }
 
