@@ -88,30 +88,36 @@ ludwig.default <- function(x,...){
 #' \code{3}: fit a discordia line using a modified maximum likelihood
 #' algorithm that includes accounts for any overdispersion by adding a
 #' geological (co)variance term.
-#' @seealso
-#' \code{\link{concordia}},
-#' \code{\link{titterington}},
-#' \code{\link{isochron}}
+#' @param anchor control parameters to fix the intercept age or common
+#'     Pb composition of the discordia fit. This is a two-element
+#'     list.
+#'
+#' \itemize{
+#'
+#' \item The first element is a boolean flag indicating whether the
+#' discordia line should be anchored. If this is \code{FALSE}, then
+#' the second item is ignored and both the common Pb composition and
+#' age are estimated.
+#'
+#' \item If the first element is \code{TRUE} and the second element is
+#' \code{NA}, then the common Pb composition is fixed at the values
+#' stored in \code{settings('iratio',...)}.
+#'
+#' item If the first element is \code{TRUE} and the second element is a
+#' number, then the discordia line is forced to intersect the
+#' concordia line at an age equal to that number.
+#' }
+#'
+#' @seealso \code{\link{concordia}}, \code{\link{titterington}},
+#'     \code{\link{isochron}}
 #' @rdname ludwig
 #' @export
-ludwig.UPb <- function(x,exterr=FALSE,alpha=0.05,model=1,...){
-    fit <- get.ta0b0(x,exterr=exterr,model=model)
+ludwig.UPb <- function(x,exterr=FALSE,alpha=0.05,model=1,
+                       anchor=list(FALSE,NA),...){
+    fit <- get.ta0b0(x,exterr=exterr,model=model,anchor=anchor)
     out <- fit[c('par','w','model')]
-    out$cov <- tryCatch({ # analytical
-        fish <- fisher.lud(x,fit=fit)
-        ns <- length(x)
-        # block inversion:
-        AA <- fish[1:ns,1:ns]
-        BB <- fish[1:ns,(ns+1):(ns+3)]
-        CC <- fish[(ns+1):(ns+3),1:ns]
-        DD <- fish[(ns+1):(ns+3),(ns+1):(ns+3)]
-        solve(DD - CC %*% solve(AA) %*% BB)
-    }, error = function(e){ # numerical
-        hessian <- stats::optimHess(fit$par,fn=LL.lud.UPb,
-                                    x=x,exterr=exterr,LL=TRUE)
-        solve(hessian)
-    })
-    mswd <- mswd.lud(fit$par,x=x,model=fit$model,w=fit$w)
+    out$cov <- fisher.lud(x,fit=fit,anchor=anchor)
+    mswd <- mswd.lud(fit$par,x=x,anchor=anchor)
     out <- c(out,mswd)
     if (model==3){
         out$w <- c(fit$w,
@@ -126,167 +132,170 @@ ludwig.UPb <- function(x,exterr=FALSE,alpha=0.05,model=1,...){
     out
 }
 
-mswd.lud <- function(ta0b0,x,model=1,w=0){
+mswd.lud <- function(ta0b0,x,anchor=list(FALSE,NA)){
     ns <- length(x)
     # Mistake in Ludwig (1998)? Multiply the following by 2?
-    SS <- LL.lud.UPb(ta0b0,x=x,exterr=FALSE,model=model,w=w,LL=FALSE)
+    SS <- LL.lud.UPb(ta0b0,x=x,exterr=FALSE,w=0,LL=FALSE)
     out <- list()
-    if (x$format<4) out$df <- ns-2
-    else out$df <- 2*ns-2
+    anchored <- anchor[[1]]
+    tanchored <- is.numeric(anchor[[2]])
+    if (x$format<4 && anchored){
+        out$df <- ns-1
+    } else if (x$format<4) {
+        out$df <- ns-2
+    } else if (anchored && tanchored){
+        out$df <- 2*ns-2
+    } else if (anchored){
+        out$df <- 2*ns-1
+    } else {
+        out$df <- 2*ns-3
+    }
     out$mswd <- as.vector(SS/out$df)
     out$p.value <- as.numeric(1-stats::pchisq(SS,out$df))
     out
 }
 
-get.ta0b0 <- function(x,exterr=FALSE,model=1){
-    init <- get.ta0b0.model2(x)
+get.ta0b0 <- function(x,exterr=FALSE,model=1,
+                      anchor=list(FALSE,NA)){
+    init <- get.ta0b0.model2(x,anchor=anchor)
     if (model==1)
-        out <- get.ta0b0.model1(x,init=init,exterr=exterr)
+        out <- get.ta0b0.model1(x,init=init,exterr=exterr,anchor=anchor)
     else if (model==2)
         out <- list(par=init,w=0)
     else if (model==3)
-        out <- get.ta0b0.model3(x,init=init,exterr=exterr)
+        out <- get.ta0b0.model3(x,init=init,exterr=exterr,anchor=anchor)
     out$model <- model
     out$exterr <- exterr
     out
 }
-get.ta0b0.model1 <- function(x,init,exterr=FALSE){
-    out <- fit_ludwig_discordia(x,init=init,w=0,model=1,exterr=exterr)
+get.ta0b0.model1 <- function(x,init,exterr=FALSE,anchor=list(FALSE,NA)){
+    out <- fit_ludwig_discordia(x,init=init,w=0,exterr=exterr,
+                                fixed=fixit(x,anchor))
     out$w <- 0
     out
 }
-# tricks the weighted regression algorithm into doing ordinary least squares
-get.ta0b0.model2 <- function(x){
+get.ta0b0.model2 <- function(x,anchor=list(FALSE,NA)){
     xy <- data2york(x,wetherill=FALSE)[,c('X','Y')]
-    xyfit <- stats::lm(xy[,'Y'] ~ xy[,'X'])
-    ta0b0 <- concordia.intersection.ab(xyfit$coef[1],xyfit$coef[2],wetherill=FALSE)$x
+    if (!anchor[[1]]) {
+        xyfit <- stats::lm(xy[,'Y'] ~ xy[,'X'])
+        intercept <- xyfit$coef[1]
+        slope <- xyfit$coef[2]
+        ta0b0 <- concordia.intersection.ab(intercept,slope,wetherill=FALSE)$x
+    } else if (is.na(anchor[[2]])){
+        if (x$format < 4){
+            b0a0 <- settings('iratio','Pb207Pb206')[1]
+        } else {
+            a0 <- settings('iratio','Pb206Pb204')[1]
+            b0 <- settings('iratio','Pb207Pb204')[1]
+            b0a0 <- b0/a0
+        }
+        intercept <- b0a0
+        xyfit <- stats::lm(I(xy[,'Y']-intercept) ~ 0 + xy[,'X'])
+        slope <- xyfit$coef
+        ta0b0 <- concordia.intersection.ab(intercept,slope,wetherill=FALSE)$x
+    } else if (is.numeric(anchor[[2]])){
+        TW <- age_to_terawasserburg_ratios(anchor[[2]],st=0,exterr=FALSE)
+        xyfit <- stats::lm(I(xy[,'Y']-TW$x['Pb207Pb206']) ~
+                           0 + I(xy[,'X']-TW$x['U238Pb206']))
+        slope <- xyfit$coef
+        intercept <- TW$x['Pb207Pb206'] - slope*TW$x['U238Pb206']
+        ta0b0 <- c(anchor[[2]],intercept)
+    }
     if (x$format>3){
         U238Pb206 <- subset(get.U238Pb206.ratios(x),select='U238Pb206')
         Pb206U238 <- subset(get.Pb206U238.ratios(x),select='Pb206U238')
         Pb204U238 <- subset(get.Pb204U238.ratios(x),select='Pb204U238')
         Pb207Pb206 <- subset(get.Pb207Pb206.ratios(x),select='Pb207Pb206')
         Pb204Pb206 <- Pb204U238/Pb206U238
-        lmfit <- stats::lm(Pb204Pb206 ~ U238Pb206)
-        ta0b0[3] <- ta0b0[2]/lmfit$coef[1] # 7/4
-        ta0b0[2] <- 1/lmfit$coef[1] # 6/4
-    }
+        if (!anchor[[1]]){
+            lmfit <- stats::lm(Pb204Pb206 ~ U238Pb206)
+            ta0b0[3] <- ta0b0[2]/lmfit$coef[1]    # 7/4
+            ta0b0[2] <- 1/lmfit$coef[1]           # 6/4
+        } else if (is.na(anchor[[2]])){
+            ta0b0[2] <- a0
+            ta0b0[3] <- b0
+        } else if (is.numeric(anchor[[2]])){
+            lmfit <- stats::lm(Pb204Pb206 ~
+                               0 + I(U238Pb206-TW$x['U238Pb206']))
+            slope <- lmfit$coef                   # 4/8
+            Pb46 <- -TW$x['U238Pb206']*slope
+            ta0b0[2] <- 1/Pb46                    # 6/4
+            ta0b0[3] <- Pb46 * TW$x['Pb207Pb206'] # 7/4
+        }
+    }    
     ta0b0
 }
-get.ta0b0.model3 <- function(x,init,exterr=FALSE){
-    fit <- fit_ludwig_discordia(x,init=init,w=0,model=1,exterr=exterr)
+get.ta0b0.model3 <- function(x,init,exterr=FALSE,anchor=list(FALSE,NA)){
+    fit <- fit_ludwig_discordia(x,init=init,w=0,exterr=exterr,
+                                fixed=fixit(x,anchor))
     ta0b0 <- fit$par
     w <- get_ludwig_disp(ta0b0,x,interval=get_lud_wrange(ta0b0,x))
-    out <- fit_ludwig_discordia(x,init=ta0b0,w=w,model=3,exterr=exterr)
+    out <- fit_ludwig_discordia(x,init=ta0b0,w=w,exterr=exterr,
+                                fixed=fixit(x,anchor))
     out$w <- w
     out
 }
-fit_ludwig_discordia <- function(x,init,w=0,model=1,exterr=FALSE){
-    stats::optim(init,fn=LL.lud.UPb,method="BFGS",x=x,
-                 model=model,w=w,exterr=exterr)
+fit_ludwig_discordia <- function(x,init,w=0,exterr=FALSE,...){
+    optifix(parms=init,fn=LL.lud.UPb,method="BFGS",x=x,
+            w=w,exterr=exterr,...)
 }
 get_ludwig_disp <- function(ta0b0,x,interval){
     stats::optimize(LL.lud.UPb.disp,interval=interval,
-                    x=x,ta0b0=ta0b0)$minimum
+                    x=x,ta0b0=ta0b0,maximum=TRUE)$maximum
 }
 get_lud_wrange <- function(ta0b0,x){
-    mswd <- mswd.lud(ta0b0,x=x,model=1,w=0)$mswd
-    c(0,sqrt(mswd)*9.74) # 9.74 = current 204/238-rati0
+    c(0,1)
 }    
 
 LL.lud.UPb.disp <- function(w,x,ta0b0){
-    LL.lud.UPb(ta0b0,x=x,exterr=FALSE,model=3,w=w,LL=TRUE)
+    LL.lud.UPb(ta0b0,x=x,exterr=FALSE,w=w,LL=TRUE)
 }
-LL.lud.UPb <- function(ta0b0,x,exterr=FALSE,model=1,w=0,LL=FALSE){
+LL.lud.UPb <- function(ta0b0,x,exterr=FALSE,w=0,LL=FALSE){
     if (x$format<4){
-        return(LL.lud.2D(ta0b0,x=x,exterr=exterr,model=model,w=w,LL=LL))
+        return(LL.lud.2D(ta0b0,x=x,exterr=exterr,w=w,LL=LL))
     } else {
-        return(LL.lud.3D(ta0b0,x=x,exterr=exterr,model=model,w=w,LL=LL))
+        return(LL.lud.3D(ta0b0,x=x,exterr=exterr,w=w,LL=LL))
     }
 }
-LL.lud.2D <- function(ta0,x,exterr=FALSE,model=1,w=0,LL=FALSE){
-    tt <- ta0[1]
-    a0 <- ta0[2]
-    l5 <- settings('lambda','U235')
-    l8 <- settings('lambda','U238')
-    U <- settings('iratio','U238U235')[1]
-    XY <- data2york(x,wetherill=FALSE)
-    b <- (exp(l5[1]*tt)-1)/U - a0*(exp(l8[1]*tt)-1) # slope
-    xy <- get.york.xy(XY,a0,b) # get adjusted xi, yi
-    ns <- length(x)
-    v <- matrix(0,1,2*ns)
-    v[1:ns] <- XY[,'X']-xy[,1]
-    v[(ns+1):(2*ns)] <- XY[,'Y']-xy[,2]
-    if (exterr){
-        Ex <- matrix(0,2*ns+2,2*ns+2)
-        Jv <- diag(1,2*ns,2*ns+2)
-    } else {
-        Ex <- matrix(0,2*ns,2*ns)
-        Jv <- diag(1,2*ns,2*ns)
+LL.lud.2D <- function(ta0,x,exterr=FALSE,w=0,LL=FALSE){
+    d <- data2ludwig(x,tt=ta0[1],a0=ta0[2],exterr=exterr,w=w)
+    R <- rbind(d$rx,d$ry)
+    out <- t(R) %*% d$omega %*% R
+    if (LL){
+        k <- length(R)
+        out <- -0.5*(out + k*log(2*pi) +
+                     determinant(2*pi*d$omegainv,logarithm=TRUE)$modulus)
     }
-    Ex[1:ns,1:ns] <- diag(XY[,'sX'])^2
-    # overdispersion added to the Pb207/Pb206-ratio:
-    Ex[(ns+1):(2*ns),(ns+1):(2*ns)] <- diag(XY[,'sY']+w)^2
-    Ex[1:ns,(ns+1):(2*ns)] <-
-        diag(XY[,'rXY'])*diag(XY[,'sX'])*diag(XY[,'sY'])
-    Ex[(ns+1):(2*ns),1:ns] <- Ex[1:ns,(ns+1):(2*ns)]
-    if (exterr){
-        Ex[2*ns+1,2*ns+1] <- l5[2]^2
-        Ex[2*ns+2,2*ns+2] <- l8[2]^2
-        Jv[1:ns,2*ns+1] <- 0 # dX/dl5
-        Jv[1:ns,2*ns+2] <-
-            -tt*exp(l8[1]*tt)/(exp(l8[1]*tt)-1)^2 # dX/dl8
-        Jv[(ns+1):(2*ns),2*ns+1] <-
-            tt*exp(l5[1]*tt)/(U*exp(l8[1]*tt)-1) # dY/dl5
-        Jv[(ns+1):(2*ns),2*ns+2] <-
-            -(exp(l5[1]*tt)-1)*tt*exp(l8[1]*tt)/ # dY/dl8
-            (U*(exp(l8[1]*tt)-1)^2)
-    }
-    E <- Jv %*% Ex %*% t(Jv)
-    SS <- v %*% solve(E) %*% t(v)
-    if (LL) out <- (determinant(E,logarithm=TRUE)$modulus+SS)/2
-    else out <- SS
     out
 }
-LL.lud.3D <- function(ta0b0,x,exterr=FALSE,model=1,w=0,LL=FALSE){
-    tt <- ta0b0[1]
-    a0 <- ta0b0[2]
-    b0 <- ta0b0[3]
-    d <- data2ludwig(x,a0=a0,b0=b0,tt=tt,
-                     exterr=exterr,model=model,w=w)
+LL.lud.3D <- function(ta0b0,x,exterr=FALSE,w=0,LL=FALSE){
+    d <- data2ludwig(x,tt=ta0b0[1],a0=ta0b0[2],
+                     b0=ta0b0[3],exterr=exterr,w=w)
     phi <- d$phi
     R <- d$R
     r <- d$r
     out <- 0
     ns <- length(d$R)
-    if (exterr){
-        omega <- d$omega
-        for (i in 1:ns){
-            i1 <- i
-            i2 <- i + ns
-            i3 <- i + 2*ns
-            for (j in 1:ns){
-                j1 <- j
-                j2 <- j + ns
-                j3 <- j + 2*ns
-                out <- out + R[i]*R[j]*omega[i1,j1] +
-                    r[i]*r[j]*omega[i2,j2] + phi[i]*phi[j]*omega[i3,j3] +
-                    2*( R[i]*r[j]*omega[i1,j2] +
+    omega <- d$omega
+    for (i in 1:ns){
+        i1 <- i
+        i2 <- i + ns
+        i3 <- i + 2*ns
+        for (j in 1:ns){
+            j1 <- j
+            j2 <- j + ns
+            j3 <- j + 2*ns
+            out <- out + R[i]*R[j]*omega[i1,j1] +
+                r[i]*r[j]*omega[i2,j2] + phi[i]*phi[j]*omega[i3,j3] +
+                2*( R[i]*r[j]*omega[i1,j2] +
                     R[i]*phi[j]*omega[i1,j3] + r[i]*phi[j]*omega[i2,j3] )
-                if (LL) out <- out +
-                    determinant(solve(omega),logarithm=TRUE)$modulus
-            }
-        }
-    } else {
-        for (i in 1:ns){
-            omega <- d$omega[[i]]
-            out <- out + omega[1,1]*R[i]^2 + omega[2,2]*r[i]^2 +
-                         omega[3,3]*phi[i]^2 + 2*( R[i]*r[i]*omega[1,2] +
-                         R[i]*phi[i]*omega[1,3] + r[i]*phi[i]*omega[2,3] )
-            if (LL) out <- out +
-                determinant(solve(omega),logarithm=TRUE)$modulus
         }
     }
-    if (LL) out <- -out/2
+    if (LL){
+        k <- 2*ns
+        out <- -0.5*(out + k*log(2*pi) +
+                     determinant(d$omegainv,logarithm=TRUE)$modulus)
+    }
     out
 }
 
@@ -294,77 +303,96 @@ fisher.lud <- function(x,...){ UseMethod("fisher.lud",x) }
 fisher.lud.default <- function(x,...){
     stop( "No default method available (yet)." )
 }
-fisher.lud.UPb <- function(x,fit,...){
-    if (x$format<4) return(fisher_lud_2D(x,fit))
-    else return(fisher_lud_3D(x,fit))
+fisher.lud.UPb <- function(x,fit,exterr=TRUE,
+                           anchor=list(FALSE,NA),...){
+    ns <- length(x)
+    if (x$format<4){
+        fish <- fisher_lud_2D(x,fit)
+        AA <- fish[1:ns,1:ns]
+        BB <- fish[1:ns,(ns+1):(ns+2)]
+        CC <- fish[(ns+1):(ns+2),1:ns]
+        DD <- fish[(ns+1):(ns+2),(ns+1):(ns+2)]
+    } else {
+        fish <- fisher_lud_3D(x,fit)
+        AA <- fish[1:ns,1:ns]
+        BB <- fish[1:ns,(ns+1):(ns+3)]
+        CC <- fish[(ns+1):(ns+3),1:ns]
+        DD <- fish[(ns+1):(ns+3),(ns+1):(ns+3)]
+    }
+    anchorfish(AA,BB,CC,DD,anchor=anchor)
+}
+anchorfish <- function(AA,BB,CC,DD,anchor){
+    npar <- nrow(DD)
+    out <- matrix(0,npar,npar)
+    if (anchor[[1]] && is.numeric(anchor[[2]])){
+        bb <- subset(BB,select=-1)
+        cc <- t(subset(t(CC),select=-1))
+        dd <- DD[-1,-1]
+        out[2:npar,2:npar] <- blockinverse(AA,bb,cc,dd)
+    } else if (anchor[[1]]){
+        bb <- subset(BB,select=1)
+        cc <- t(subset(t(CC),select=1))
+        dd <- DD[1,1]
+        out[1,1] <- blockinverse(AA,bb,cc,dd)
+    } else {
+        out <- blockinverse(AA,BB,CC,DD)
+    }
+    out
 }
 fisher_lud_2D <- function(x,fit){
-    stop('not implemented yet')
+    tt <- fit$par[1]
+    a0 <- fit$par[2]
+    d <- data2ludwig_2D(x,tt=tt,a0=a0,w=0,exterr=fit$exterr)
+    l5 <- settings('lambda','U235')[1]
+    l8 <- settings('lambda','U238')[1]
+    U <- settings('iratio','U238U235')[1]
+    ns <- length(x)
+    ones <- matrix(1,ns,1)
+    B <-  (exp(l5*tt)-1)/U - a0*(exp(l8*tt)-1)
+    C <- -t(d$X)%*%d$omega12 + t(d$x)%*%d$omega12 + t(d$x)%*%d$omega21 -
+        t(d$Y)%*%d$omega22 + t(ones)%*%d$omega22*a0 + 2*t(d$x)%*%d$omega22*B
+    D <- -t(d$X)%*%d$omega12%*%d$x + t(d$x)%*%d$omega12%*%d$x -
+        t(d$Y)%*%d$omega22%*%d$x + t(ones)%*%d$omega22%*%d$x*a0 -
+        t(d$x)%*%d$omega21%*%d$X + t(d$x)%*%d$omega21%*%d$x -
+        t(d$x)%*%d$omega22%*%d$Y + t(d$x)%*%d$omega22%*%ones*a0 +
+        2*t(d$x)%*%d$omega22%*%d$x*B
+    dBdt <- l5*exp(l5*tt)/U - a0*exp(l8*tt)*l8
+    dBda0 <- -(exp(l8*tt)-1)
+    d2Bdtda0 <- exp(l8*tt)*l8
+    d2Bda0dt <- exp(l8*tt)*l8
+    d2Bdt2 <- (l5^2)*exp(l5*tt)/U - a0*exp(l8*tt)*(l8^2)
+    d2Bda02 <- 0
+    dDda0 <- t(ones)%*%d$omega22%*%d$x +
+        t(d$x)%*%d$omega22%*%ones + 2*t(d$x)%*%d$omega22%*%d$x*dBda0
+    ns <- length(x)
+    out <- matrix(0,ns+2,ns+2)
+    out[1:ns,1:ns] <- # d2S/dx2
+        2*(d$omega11 + d$omega21*B + d$omega12*B + d$omega22*B^2)
+
+    out[ns+2,1:ns] <-   # d2S/dxda0
+        2*(C*dBda0 + t(ones)%*%d$omega21 + t(ones)%*%d$omega22*B)
+    out[ns+2,ns+2] <- # d2S/da02
+        2*(t(ones)%*%d$omega22%*%ones +
+           t(ones)%*%d$omega22%*%d$x*dBda0 +
+           t(d$x)%*%d$omega22%*%ones*dBda0 +
+           t(d$x)%*%d$omega22%*%d$x*(dBda0^2))
+    out[ns+1,1:ns] <- 2*C*dBdt # d2S/dxdt
+    out[ns+1,ns+1] <- # d2S/dt2
+        D*d2Bdt2 + 2*t(d$x)%*%d$omega22%*%d$x*(dBdt^2)
+    out[ns+1,ns+2] <- # d2S/dt0da0
+        D*d2Bdtda0 + dDda0*dBdt
+    out[1:ns,ns+1] <- t(out[ns+1,1:ns]) # d2S/dtdx
+    out[1:ns,ns+2] <- t(out[ns+2,1:ns]) # d2S/da0dx
+    out[ns+2,ns+1] <- out[ns+1,ns+2]
+    out/2
 }
 fisher_lud_3D <- function(x,fit){
     tt <- fit$par[1]
     a0 <- fit$par[2]
     b0 <- fit$par[3]
-    d <- data2ludwig(x,a0=a0,b0=b0,tt=tt,z=z,exterr=fit$exterr,
-                     model=fit$model,w=fit$w)
+    d <- data2ludwig_3D(x,tt=tt,a0=a0,b0=b0,w=0,exterr=fit$exterr)
     z <- d$z
     omega <- d$omega
-    if (fit$exterr)
-        out <- fisher_lud_with_decay_err(tt,a0=a0,b0=b0,z=z,omega=omega)
-    else
-        out <- fisher_lud_without_decay_err(tt,a0=a0,b0=b0,z=z,omega=omega)
-    out
-}
-fisher_lud_without_decay_err <- function(tt,a0,b0,z,omega){
-    ns <- length(z)
-    l5 <- settings('lambda','U235')
-    l8 <- settings('lambda','U238')
-    U <- settings('iratio','U238U235')[1]
-    Q235 <- l5[1]*exp(l5[1]*tt)
-    Q238 <- l8[1]*exp(l8[1]*tt)
-    d2L.dt2 <- 0
-    d2L.da02 <- 0
-    d2L.db02 <- 0
-    d2L.da0dt <- 0
-    d2L.db0dt <- 0
-    d2L.da0db0 <- 0
-    out <- matrix(0,ns+3,ns+3)
-    for (i in 1:ns){
-        O <- omega[[i]]
-        d2L.dt2 <- d2L.dt2 +
-                   O[1,1]*Q235^2 + O[2,2]*Q238^2 + 2*Q235*Q238*O[1,2]
-        d2L.da02 <- d2L.da02 + O[2,2]*z[i]^2
-        d2L.db02 <- d2L.db02 + O[1,1]*(U*z[i])^2
-        d2L.da0dt <- d2L.da0dt + z[i]*(Q238*O[2,2] + Q235*O[1,2])
-        d2L.db0dt <- d2L.db0dt + U*z[i]*(Q235*O[1,1] + Q238*O[1,2])
-        d2L.da0db0 <- d2L.da0db0 + U*O[1,2]*z[i]^2
-        d2L.dz2 <- O[1,1]*(U*b0)^2 + O[2,2]*a0^2 + O[3,3] +
-                   2*(a0*U*b0*O[1,2] + U*b0*O[1,3] + a0*O[3,3])
-        d2L.dzdt <- Q235*(U*b0*O[1,1] + O[1,3]) +
-                    Q238*(a0*O[2,2]+O[2,3]) +
-                   (Q238*U*b0 + Q235*a0)*O[1,2]
-        d2L.dzda0 <- z[i]*(a0*O[2,2] + U*b0*O[1,2] + O[2,3])
-        d2L.dzdb0 <- U*z[i]*(a0*O[1,2] + U*b0*O[1,1] + O[1,3])
-        out[i,i] <- d2L.dz2
-        out[i,ns+1] <- d2L.dzdt
-        out[i,ns+2] <- d2L.dzda0
-        out[i,ns+3] <- d2L.dzdb0
-        out[ns+1,i] <- out[i,ns+1]
-        out[ns+2,i] <- out[i,ns+2]
-        out[ns+3,i] <- out[i,ns+3]
-    }
-    out[ns+1,ns+1] <- d2L.dt2
-    out[ns+2,ns+2] <- d2L.da02
-    out[ns+3,ns+3] <- d2L.db02
-    out[ns+1,ns+2] <- d2L.da0dt
-    out[ns+1,ns+3] <- d2L.db0dt
-    out[ns+2,ns+3] <- d2L.da0db0
-    out[ns+2,ns+1] <- out[ns+1,ns+2]
-    out[ns+3,ns+1] <- out[ns+1,ns+3]
-    out[ns+3,ns+2] <- out[ns+2,ns+3]
-    out
-}
-fisher_lud_with_decay_err <- function(tt,a0,b0,z,omega){
     ns <- length(z)
     l5 <- settings('lambda','U235')
     l8 <- settings('lambda','U238')
@@ -442,61 +470,62 @@ fisher_lud_with_decay_err <- function(tt,a0,b0,z,omega){
 
 data2ludwig <- function(x,...){ UseMethod("data2ludwig",x) }
 data2ludwig.default <- function(x,...){ stop('default function undefined') }
-data2ludwig.UPb <- function(x,a0,b0,tt,exterr=FALSE,model=1,w=0,...){
-    if (exterr)
-        out <- data2ludwig_with_decay_err(x,a0=a0,b0=b0,tt=tt)
+data2ludwig.UPb <- function(x,tt,a0,b0=0,exterr=FALSE,w=0,...){
+    if (x$format<4)
+        out <- data2ludwig_2D(x,tt=tt,a0=a0,w=w,exterr=exterr)  
     else
-        out <- data2ludwig_without_decay_err(x,a0=a0,b0=b0,tt=tt,
-                                             model=model,w=w)
+        out <- data2ludwig_3D(x,tt=tt,a0=a0,b0=b0,w=w,exterr=exterr)
     out
 }
-data2ludwig_without_decay_err <- function(x,a0,b0,tt,model=1,w=0){
-    # initialise:
+data2ludwig_2D <- function(x,tt,a0,w=0,exterr=FALSE){
     l5 <- settings('lambda','U235')
     l8 <- settings('lambda','U238')
     U <- settings('iratio','U238U235')[1]
     ns <- length(x)
-    R <- rep(0,ns)
-    r <- rep(0,ns)
-    phi <- rep(0,ns)
-    Z <- rep(0,ns)
-    z <- rep(0,ns)
-    omega <- list()
-    E <- matrix(0,3,3)
-    Zbar <- mean(get.Pb204U238.ratios(x))
-    Ybar <- a0*Zbar
-    Xbar <- U*b0*Zbar
+    B <-  (exp(l5[1]*tt)-1)/U - a0*(exp(l8[1]*tt)-1)
+    dBdl5 <- tt*exp(l5[1]*tt)/U
+    dBdl8 <- -a0*tt*exp(l8[1]*tt)
+    E <- matrix(0,2*ns+2,2*ns+2)
+    J <- matrix(0,2*ns,2*ns+2)
+    X <- matrix(0,ns,1)
+    Y <- matrix(0,ns,1)
     for (i in 1:ns){
-        d <- wetherill(x,i=i,exterr=FALSE)
-        Z[i] <- d$x['Pb204U238']
-        R[i] <- d$x['Pb207U235'] - exp(l5[1]*tt) + 1 - U*b0*Z[i]
-        r[i] <- d$x['Pb206U238'] - exp(l8[1]*tt) + 1 - a0*Z[i]
-        # overdispersion applied proportional to average composition
-        E[1,1] <- d$cov['Pb207U235','Pb207U235'] + (Xbar*w)^2
-        E[2,2] <- d$cov['Pb206U238','Pb206U238'] + (Ybar*w)^2
-        E[3,3] <- d$cov['Pb204U238','Pb204U238'] + (Zbar*w)^2
-        E[1,2] <- d$cov['Pb207U235','Pb206U238']
-        E[1,3] <- d$cov['Pb207U235','Pb204U238']
-        E[2,3] <- d$cov['Pb206U238','Pb204U238']
-        E[2,1] <- E[1,2]
-        E[3,1] <- E[1,3]
-        E[3,2] <- E[2,3]
-        O <- solve(E)
-        omega[[i]] <- O
-        # rearrange sum of squares:
-        AA <- O[1,1]*(U*b0)^2 + O[2,2]*a0^2 + O[3,3] +
-            2*(U*a0*b0*O[1,2] + U*b0*O[1,3] + a0*O[2,3])
-        BB <- R[i]*U*b0*O[1,1] + r[i]*a0*O[2,2] +
-             (R[i]*a0 + U*b0*r[i])*O[1,2] +
-            R[i]*O[1,3] + r[i]*O[2,3]
-        CC <- O[1,1]*R[i]^2 + O[2,2]*r[i]^2 + 2*R[i]*r[i]*O[1,2]
-        z[i] <- Z[i] + BB/AA
-        phi[i] <- Z[i] - z[i]
+        XY <- tera.wasserburg(x,i,exterr=FALSE)
+        X[i] <- XY$x['U238Pb206']
+        Y[i] <- XY$x['Pb207Pb206']
+        E[(2*i-1):(2*i),(2*i-1):(2*i)] <- XY$cov
+        E[2*i,2*i] <- E[2*i,2*i] + (a0*w)^2
+        J[i,2*i-1] <- 1 # drx/dX
+        J[ns+i,2*i] <- 1 # dry/dY
+        if (exterr){
+            J[ns+i,2*ns+1] <- -dBdl5*X[i]
+            J[ns+i,2*ns+2] <- -dBdl8*X[i]
+        }
     }
-    out <- list(R=R,r=r,phi=phi,z=z,omega=omega)
+    E[2*ns+1,2*ns+1] <- l5[2]^2
+    E[2*ns+2,2*ns+2] <- l8[2]^2
+    ER <- J %*% E %*% t(J)
+    out <- list()
+    omega <- solve(ER)
+    omega11 <- omega[1:ns,1:ns]
+    omega12 <- omega[1:ns,(ns+1):(2*ns)]
+    omega21 <- omega[(ns+1):(2*ns),1:ns]
+    omega22 <- omega[(ns+1):(2*ns),(ns+1):(2*ns)]
+    out$x <- solve(omega11 + B*(omega12+omega21) + omega22*B^2) %*%
+        (omega11%*%X + B*omega12%*%X + omega21%*%(Y-a0) + B*omega22%*%(Y-a0))
+    out$omegainv <- ER
+    out$omega <- omega
+    out$omega11 <- omega11
+    out$omega12 <- omega12
+    out$omega21 <- omega21
+    out$omega22 <- omega22
+    out$X <- X
+    out$Y <- Y
+    out$rx <- X - out$x
+    out$ry <- Y - a0 - B*out$x
+    out
 }
-data2ludwig_with_decay_err <- function(x,a0,b0,tt,w=0){
-    # initialise:
+data2ludwig_3D <- function(x,tt,a0,b0,w=0,exterr=FALSE){
     l5 <- settings('lambda','U235')
     l8 <- settings('lambda','U238')
     U <- settings('iratio','U238U235')[1]
@@ -505,10 +534,7 @@ data2ludwig_with_decay_err <- function(x,a0,b0,tt,w=0){
     r <- rep(0,ns)
     Z <- rep(0,ns)
     E <- matrix(0,3*ns,3*ns)
-    Zbar <- mean(get.Pb204U238.ratios(x))
-    Ybar <- a0*Zbar
-    Xbar <- U*b0*Zbar
-    if (TRUE){ # TRUE if propagating decay errors
+    if (exterr){
         P235 <- tt*exp(l5[1]*tt)
         P238 <- tt*exp(l8[1]*tt)
         E[1:ns,1:ns] <- (P235*l5[2])^2 # A
@@ -519,11 +545,12 @@ data2ludwig_with_decay_err <- function(x,a0,b0,tt,w=0){
         Z[i] <- d$x['Pb204U238']
         R[i] <- d$x['Pb207U235'] - exp(l5[1]*tt) + 1 - U*b0*Z[i]
         r[i] <- d$x['Pb206U238'] - exp(l8[1]*tt) + 1 - a0*Z[i]
-        E[i,i] <- E[i,i] + d$cov['Pb207U235','Pb207U235'] + (Xbar*w)^2 # A
-        E[ns+i,ns+i] <- E[ns+i,ns+i] + d$cov['Pb206U238','Pb206U238'] + (Ybar*w)^2 # B
-        E[2*ns+i,2*ns+i] <- d$cov['Pb204U238','Pb204U238'] + (Zbar*w)^2 # C
-        E[i,ns+i] <- d$cov['Pb207U235','Pb206U238'] # D
-        E[ns+i,i] <- E[i,ns+i]
+        Ew <- get.Ew(w=w,Z=Z[i],a0=a0,b0=b0,U=U)
+        E[i,i] <- E[i,i] + d$cov['Pb207U235','Pb207U235'] + Ew[1,1] # A
+        E[ns+i,ns+i] <- E[ns+i,ns+i] + d$cov['Pb206U238','Pb206U238'] + Ew[2,2] # B
+        E[2*ns+i,2*ns+i] <- d$cov['Pb204U238','Pb204U238'] # C
+        E[i,ns+i] <- d$cov['Pb207U235','Pb206U238'] + Ew[1,2] # D
+        E[ns+i,i] <- E[i,ns+i] + Ew[2,1]
         E[i,2*ns+i] <- d$cov['Pb207U235','Pb204U238'] # E
         E[2*ns+i,i] <- E[i,2*ns+i]
         E[ns+i,2*ns+i] <- d$cov['Pb206U238','Pb204U238'] # F
@@ -550,5 +577,30 @@ data2ludwig_with_decay_err <- function(x,a0,b0,tt,w=0){
     }
     phi <- solve(V,W)
     z <- Z - phi
-    out <- list(R=R,r=r,phi=phi,z=z,omega=omega)
+    out <- list(R=R,r=r,phi=phi,z=z,omega=omega,omegainv=E)
+}
+get.Ew <- function(w,Z,a0,b0,U){
+    J <- matrix(0,2,4)
+    J[1,1] <- Z*U
+    J[1,3] <- a0*U
+    J[1,4] <- a0*Z
+    J[2,2] <- Z
+    J[2,3] <- b0
+    E <- matrix(0,4,4)
+    E[1,1] <- (a0*w)^2
+    E[2,2] <- (b0*w)^2
+    J %*% E %*% t(J)
+}
+
+fixit <- function(x,anchor=list(FALSE,NA)){
+    if (x$format<4){
+        if (!anchor[[1]]) out <- rep(FALSE,2)
+        else if (is.numeric(anchor[[2]])) out <- c(TRUE,FALSE)
+        else out <- c(FALSE,TRUE)
+    } else {
+        if (!anchor[[1]]) out <- rep(FALSE,3)
+        else if (is.numeric(anchor[[2]])) out <- c(TRUE,FALSE,FALSE)
+        else out <- c(FALSE,TRUE,TRUE)
+    }
+    out
 }
