@@ -96,11 +96,12 @@
 #' included into the weighted mean calculation}
 #'
 #' \item{plotpar}{list of plot parameters for the weighted mean
-#' diagram}
-#' }
+#' diagram} }
 #' @rdname weightedmean
 #' @export
-weightedmean <- function(x,...){ UseMethod("weightedmean",x) }
+weightedmean <- function(x,...){
+    UseMethod("weightedmean",x)
+}
 #' @param detect.outliers logical flag indicating whether outliers
 #'     should be detected and rejected using Chauvenet's Criterion.
 #' @param plot logical flag indicating whether the function should
@@ -110,8 +111,13 @@ weightedmean <- function(x,...){ UseMethod("weightedmean",x) }
 #'     plot automatically.
 #' @param to maximum y-axis limit. Setting \code{to=NA} scales the
 #'     plot automatically.
-#' @param rect.col the fill colour of the rectangles used to show the
-#'     measurements or age estimates.
+#' @param levels a vector with additional values to be displayed as
+#'     different background colours of the plot symbols.
+#' @param clabel label of the colour legend
+#' @param rect.col a vector of two fill colours used to show the
+#'     measurements or age estimates.  If \code{levels=NA}, then only
+#'     the first colour is used. If \code{levels} is a vector of
+#'     numbers, then \code{bg} is used to construct a colour ramp.
 #' @param outlier.col if \code{detect.outliers=TRUE}, the outliers are
 #'     given a different colour.
 #' @param sigdig the number of significant digits of the numerical
@@ -129,36 +135,33 @@ weightedmean <- function(x,...){ UseMethod("weightedmean",x) }
 #' @export
 weightedmean.default <- function(x,from=NA,to=NA,random.effects=TRUE,
                                  detect.outliers=TRUE,plot=TRUE,
-                                 rect.col=rgb(0,1,0,0.5),
-                                 outlier.col=rgb(0,1,1,0.5),sigdig=2,
+                                 levels=NA,clabel="",
+                                 rect.col=c("#00FF0080","#FF000080"),
+                                 outlier.col="#00FFFF80",sigdig=2,
                                  alpha=0.05,ranked=FALSE,hide=NULL,
                                  omit=NULL,omit.col=NA,...){
     ns <- nrow(x)
-    plotit <- (1:ns)%ni%hide
     calcit <- (1:ns)%ni%c(hide,omit)
-    X <- x[plotit,1]
-    sX <- x[plotit,2]
-    omit <- omit[plotit]
-    calcit <- calcit[plotit]
+    X <- x[,1]
+    sX <- x[,2]
     valid <- !is.na(X) & !is.na(sX) & calcit
     nvalid <- count(valid)
     if (detect.outliers){
         while (TRUE){
             valid <- chauvenet(X,sX,valid=valid,
                                random.effects=random.effects)
-            if (count(valid) < nvalid)
-                nvalid <- count(valid)
-            else
-                break
+            if (count(valid) < nvalid){ nvalid <- count(valid) }
+            else{ break }
         }
     }
     out <- get.weightedmean(X,sX,random.effects=random.effects,
                             valid=valid,alpha=alpha)
     if (plot){
-        plot_weightedmean(X,sX,fit=out,from=from,to=to,
-                          rect.col=rect.col,outlier.col=outlier.col,
-                          sigdig=sigdig,alpha=alpha,ranked=ranked,
-                          hide=hide,omit=omit,omit.col=omit.col,...)
+        plot_weightedmean(X,sX,fit=out,from=from,to=to,levels=levels,
+                          clabel=clabel,rect.col=rect.col,
+                          outlier.col=outlier.col,sigdig=sigdig,
+                          alpha=alpha,ranked=ranked,hide=hide,
+                          omit=omit,omit.col=omit.col,...)
     }
     invisible(out)
 }
@@ -204,8 +207,9 @@ weightedmean.default <- function(x,from=NA,to=NA,random.effects=TRUE,
 #' @export
 weightedmean.UPb <- function(x,random.effects=TRUE,
                              detect.outliers=TRUE,plot=TRUE,
-                             from=NA,to=NA,rect.col=rgb(0,1,0,0.5),
-                             outlier.col=rgb(0,1,1,0.5),sigdig=2,
+                             from=NA,to=NA,levels=NA,clabel="",
+                             rect.col=c("#00FF0080","#FF000080"),
+                             outlier.col="#00FFFF80",sigdig=2,
                              type=4,cutoff.76=1100,
                              cutoff.disc=c(-15,5),alpha=0.05,
                              exterr=TRUE,ranked=FALSE,common.Pb=0,
@@ -216,19 +220,21 @@ weightedmean.UPb <- function(x,random.effects=TRUE,
         X <- x
     weightedmean_helper(X,random.effects=random.effects,
                         detect.outliers=detect.outliers,plot=plot,
-                        from=from,to=to,rect.col=rect.col,
-                        outlier.col=outlier.col,type=type,
-                        cutoff.76=cutoff.76,cutoff.disc=cutoff.disc,
-                        sigdig=sigdig,alpha=alpha,exterr=exterr,
-                        units='Ma',ranked=ranked,hide=hide,
-                        omit=omit,omit.col=omit.col,...)
+                        from=from,to=to,levels=levels,clabel=clabel,
+                        rect.col=rect.col,outlier.col=outlier.col,
+                        type=type,cutoff.76=cutoff.76,
+                        cutoff.disc=cutoff.disc,sigdig=sigdig,
+                        alpha=alpha,exterr=exterr,units='Ma',
+                        ranked=ranked,hide=hide, omit=omit,
+                        omit.col=omit.col,...)
 }
 #' @rdname weightedmean
 #' @export
 weightedmean.PbPb <- function(x,random.effects=TRUE,
-                              detect.outliers=TRUE,plot=TRUE,
-                              from=NA,to=NA,rect.col=rgb(0,1,0,0.5),
-                              outlier.col=rgb(0,1,1,0.5),sigdig=2,
+                              detect.outliers=TRUE,plot=TRUE, from=NA,
+                              to=NA,levels=NA,clabel="",
+                              rect.col=c("#00FF0080","#FF000080"),
+                              outlier.col="#00FFFF80",sigdig=2,
                               alpha=0.05,exterr=TRUE,common.Pb=1,
                               ranked=FALSE,hide=NULL,omit=NULL,
                               omit.col=NA,...){
@@ -238,10 +244,10 @@ weightedmean.PbPb <- function(x,random.effects=TRUE,
         X <- x
     weightedmean_helper(X,random.effects=random.effects,
                         detect.outliers=detect.outliers,plot=plot,
-                        from=from,to=to,rect.col=rect.col,
-                        outlier.col=outlier.col,sigdig=sigdig,
-                        alpha=alpha,exterr=exterr,units='Ma',
-                        ranked=ranked,hide=hide,omit=omit,
+                        from=from,to=to,levels=levels,clabel=clabel,
+                        rect.col=rect.col,outlier.col=outlier.col,
+                        sigdig=sigdig, alpha=alpha,exterr=exterr,
+                        units='Ma', ranked=ranked,hide=hide,omit=omit,
                         omit.col=omit.col,...)
 }
 #' @param i2i `isochron to intercept': calculates the initial (aka
@@ -278,129 +284,138 @@ weightedmean.PbPb <- function(x,random.effects=TRUE,
 #' @rdname weightedmean
 #' @export
 weightedmean.ThU <- function(x,random.effects=TRUE,
-                             detect.outliers=TRUE,plot=TRUE,
-                             from=NA,to=NA,rect.col=rgb(0,1,0,0.5),
-                             outlier.col=rgb(0,1,1,0.5),sigdig=2,
+                             detect.outliers=TRUE,plot=TRUE, from=NA,
+                             to=NA,levels=NA,clabel="",
+                             rect.col=c("#00FF0080","#FF000080"),
+                             outlier.col="#00FFFF80",sigdig=2,
                              alpha=0.05,ranked=FALSE,i2i=TRUE,
                              detritus=0,Th02=c(0,0),
                              Th02U48=c(0,0,1e6,0,0,0,0,0,0),
                              hide=NULL,omit=NULL,omit.col=NA,...){
     weightedmean_helper(x,random.effects=random.effects,
                         detect.outliers=detect.outliers,plot=plot,
-                        from=from,to=to,rect.col=rect.col,
-                        outlier.col=outlier.col,sigdig=sigdig,
-                        alpha=alpha,ranked=ranked,i2i=i2i,units='ka',
-                        detritus=detritus,Th02=Th02,Th02U48=Th02U48,
-                        hide=hide,omit=omit,omit.col=omit.col,...)
+                        from=from,to=to,levels=levels,clabel=clabel,
+                        rect.col=rect.col,outlier.col=outlier.col,
+                        sigdig=sigdig,alpha=alpha,ranked=ranked,
+                        i2i=i2i,units='ka',detritus=detritus,
+                        Th02=Th02,Th02U48=Th02U48,hide=hide,
+                        omit=omit,omit.col=omit.col,...)
 }
 #' @rdname weightedmean
 #' @export
 weightedmean.ArAr <- function(x,random.effects=TRUE,
-                              detect.outliers=TRUE,plot=TRUE,
-                              from=NA,to=NA,rect.col=rgb(0,1,0,0.5),
-                              outlier.col=rgb(0,1,1,0.5),sigdig=2,
+                              detect.outliers=TRUE,plot=TRUE, from=NA,
+                              to=NA,levels=NA,clabel="",
+                              rect.col=c("#00FF0080","#FF000080"),
+                              outlier.col="#00FFFF80",sigdig=2,
                               alpha=0.05,exterr=TRUE,ranked=FALSE,
                               i2i=FALSE,hide=NULL,omit=NULL,
                               omit.col=NA,...){
     weightedmean_helper(x,random.effects=random.effects,
                         detect.outliers=detect.outliers,plot=plot,
-                        from=from,to=to,rect.col=rect.col,
-                        outlier.col=outlier.col,sigdig=sigdig,
-                        alpha=alpha,exterr=exterr,i2i=i2i,
-                        units='Ma',ranked=ranked,hide=hide,
+                        from=from,to=to,levels=levels,clabel=clabel,
+                        rect.col=rect.col,outlier.col=outlier.col,
+                        sigdig=sigdig,alpha=alpha,exterr=exterr,
+                        i2i=i2i,units='Ma',ranked=ranked,hide=hide,
                         omit=omit,omit.col=omit.col,...)
 }
 #' @rdname weightedmean
 #' @export
 weightedmean.KCa <- function(x,random.effects=TRUE,
-                             detect.outliers=TRUE,plot=TRUE,
-                             from=NA,to=NA,rect.col=rgb(0,1,0,0.5),
-                             outlier.col=rgb(0,1,1,0.5),sigdig=2,
+                             detect.outliers=TRUE,plot=TRUE, from=NA,
+                             to=NA,levels=NA,clabel="",
+                             rect.col=c("#00FF0080","#FF000080"),
+                             outlier.col="#00FFFF80",sigdig=2,
                              alpha=0.05,exterr=TRUE,ranked=FALSE,
                              i2i=FALSE,hide=NULL,omit=NULL,
                              omit.col=NA,...){
     weightedmean_helper(x,random.effects=random.effects,
                         detect.outliers=detect.outliers,plot=plot,
-                        from=from,to=to,rect.col=rect.col,
-                        outlier.col=outlier.col,sigdig=sigdig,
-                        alpha=alpha,exterr=exterr,i2i=i2i,
-                        units='Ma',ranked=ranked,hide=NULL,
+                        from=from,to=to,levels=levels,clabel=clabel,
+                        rect.col=rect.col,outlier.col=outlier.col,
+                        sigdig=sigdig,alpha=alpha,exterr=exterr,
+                        i2i=i2i,units='Ma',ranked=ranked,hide=NULL,
                         omit=omit,omit.col=omit.col,...)
 }
 #' @rdname weightedmean
 #' @export
 weightedmean.ReOs <- function(x,random.effects=TRUE,
-                              detect.outliers=TRUE,plot=TRUE,
-                              from=NA,to=NA,rect.col=rgb(0,1,0,0.5),
-                              outlier.col=rgb(0,1,1,0.5),sigdig=2,
+                              detect.outliers=TRUE,plot=TRUE, from=NA,
+                              to=NA,levels=NA,clabel="",
+                              rect.col=c("#00FF0080","#FF000080"),
+                              outlier.col="#00FFFF80",sigdig=2,
                               alpha=0.05,exterr=TRUE,ranked=FALSE,
                               i2i=TRUE,hide=NULL,omit=NULL,
                               omit.col=NA,...){
     weightedmean_helper(x,random.effects=random.effects,
                         detect.outliers=detect.outliers,plot=plot,
-                        from=from,to=to,rect.col=rect.col,
-                        outlier.col=outlier.col,sigdig=sigdig,
-                        alpha=alpha,exterr=exterr,i2i=i2i,
-                        units='Ma',ranked=ranked,hide=hide,
+                        from=from,to=to,levels=levels,clabel=clabel,
+                        rect.col=rect.col,outlier.col=outlier.col,
+                        sigdig=sigdig, alpha=alpha,exterr=exterr,
+                        i2i=i2i,units='Ma',ranked=ranked,hide=hide,
                         omit=omit,omit.col=omit.col,...)
 }
 #' @rdname weightedmean
 #' @export
 weightedmean.SmNd <- function(x,random.effects=TRUE,
-                              detect.outliers=TRUE,plot=TRUE,
-                              from=NA,to=NA,rect.col=rgb(0,1,0,0.5),
-                              outlier.col=rgb(0,1,1,0.5),sigdig=2,
+                              detect.outliers=TRUE,plot=TRUE,from=NA,
+                              to=NA,levels=NA,clabel="",
+                              rect.col=c("#00FF0080","#FF000080"),
+                              outlier.col="#00FFFF80",sigdig=2,
                               alpha=0.05,exterr=TRUE,ranked=FALSE,
                               i2i=TRUE,hide=NULL,omit=NULL,
                               omit.col=NA,...){
     weightedmean_helper(x,random.effects=random.effects,
                         detect.outliers=detect.outliers,plot=plot,
-                        from=from,to=to,rect.col=rect.col,
-                        outlier.col=outlier.col,sigdig=sigdig,
-                        alpha=alpha,exterr=exterr,i2i=i2i,
-                        units='Ma',ranked=ranked,hide=hide,
+                        from=from,to=to,levels=levels,clabel=clabel,
+                        rect.col=rect.col,outlier.col=outlier.col,
+                        sigdig=sigdig,alpha=alpha,exterr=exterr,
+                        i2i=i2i,units='Ma',ranked=ranked,hide=hide,
                         omit=omit,omit.col=omit.col,...)
 }
 #' @rdname weightedmean
 #' @export
 weightedmean.RbSr <- function(x,random.effects=TRUE,
-                              detect.outliers=TRUE,plot=TRUE,
-                              from=NA,to=NA,rect.col=rgb(0,1,0,0.5),
-                              outlier.col=rgb(0,1,1,0.5),sigdig=2,
-                              alpha=0.05,exterr=TRUE,
-                              i2i=TRUE,ranked=FALSE,hide=NULL,
-                              omit=NULL,omit.col=NA,...){
-    weightedmean_helper(x,random.effects=random.effects,
-                        detect.outliers=detect.outliers,plot=plot,
-                        from=from,to=to,rect.col=rect.col,
-                        outlier.col=outlier.col,sigdig=sigdig,
-                        alpha=alpha,exterr=exterr,i2i=i2i,
-                        units='Ma',ranked=ranked,hide=hide,
-                        omit=omit,omit.col=omit.col,...)
-}
-#' @rdname weightedmean
-#' @export
-weightedmean.LuHf <- function(x,random.effects=TRUE,
-                              detect.outliers=TRUE,plot=TRUE,
-                              from=NA,to=NA,rect.col=rgb(0,1,0,0.5),
-                              outlier.col=rgb(0,1,1,0.5),sigdig=2,
+                              detect.outliers=TRUE,plot=TRUE,from=NA,
+                              to=NA,levels=NA,clabel="",
+                              rect.col=c("#00FF0080","#FF000080"),
+                              outlier.col="#00FFFF80",sigdig=2,
                               alpha=0.05,exterr=TRUE,i2i=TRUE,
                               ranked=FALSE,hide=NULL,omit=NULL,
                               omit.col=NA,...){
     weightedmean_helper(x,random.effects=random.effects,
                         detect.outliers=detect.outliers,plot=plot,
-                        from=from,to=to,rect.col=rect.col,
-                        outlier.col=outlier.col,sigdig=sigdig,
-                        alpha=alpha,exterr=exterr,i2i=i2i,
-                        units='Ma',ranked=ranked,hide=hide,
+                        from=from,to=to,levels=levels,clabel=clabel,
+                        rect.col=rect.col,outlier.col=outlier.col,
+                        sigdig=sigdig,alpha=alpha,exterr=exterr,
+                        i2i=i2i,units='Ma',ranked=ranked,hide=hide,
+                        omit=omit,omit.col=omit.col,...)
+}
+#' @rdname weightedmean
+#' @export
+weightedmean.LuHf <- function(x,random.effects=TRUE,
+                              detect.outliers=TRUE,plot=TRUE,from=NA,
+                              to=NA,levels=NA,clabel="",
+                              rect.col=c("#00FF0080","#FF000080"),
+                              outlier.col="#00FFFF80",sigdig=2,
+                              alpha=0.05,exterr=TRUE,i2i=TRUE,
+                              ranked=FALSE,hide=NULL,omit=NULL,
+                              omit.col=NA,...){
+    weightedmean_helper(x,random.effects=random.effects,
+                        detect.outliers=detect.outliers,plot=plot,
+                        from=from,to=to,levels=levels,clabel=clabel,
+                        rect.col=rect.col,outlier.col=outlier.col,
+                        sigdig=sigdig,alpha=alpha,exterr=exterr,
+                        i2i=i2i,units='Ma',ranked=ranked,hide=hide,
                         omit=omit,omit.col=omit.col,...)
 }
 #' @rdname weightedmean
 #' @export
 weightedmean.UThHe <- function(x,random.effects=TRUE,
                                detect.outliers=TRUE,plot=TRUE,
-                               from=NA,to=NA,rect.col=rgb(0,1,0,0.5),
-                               outlier.col=rgb(0,1,1,0.5),sigdig=2,
+                               from=NA,to=NA,levels=NA,clabel="",
+                               rect.col=c("#00FF0080","#FF000080"),
+                               outlier.col="#00FFFF80",sigdig=2,
                                alpha=0.05,ranked=FALSE,hide=NULL,
                                omit=NULL,omit.col=NA,...){
     tt <- UThHe.age(x)
@@ -409,6 +424,7 @@ weightedmean.UThHe <- function(x,random.effects=TRUE,
                                 omit=omit)
     if (plot){
         plot_weightedmean(tt[,1],tt[,2],fit=fit,from=from,to=to,
+                          levels=levels,clabel=clabel,
                           rect.col=rect.col,outlier.col=outlier.col,
                           sigdig=sigdig,alpha=alpha,units='Ma',
                           ranked=ranked,hide=hide,omit=omit,
@@ -420,11 +436,13 @@ weightedmean.UThHe <- function(x,random.effects=TRUE,
 #' @export
 weightedmean.fissiontracks <- function(x,random.effects=TRUE,
                                        detect.outliers=TRUE,plot=TRUE,
-                                       from=NA,to=NA,rect.col=rgb(0,1,0,0.5),
-                                       outlier.col=rgb(0,1,1,0.5),
+                                       from=NA,to=NA,levels=NA,clabel="",
+                                       rect.col=c("#00FF0080","#FF000080"),
+                                       outlier.col="#00FFFF80",
                                        sigdig=2,alpha=0.05,
-                                       exterr=TRUE,ranked=FALSE,hide=NULL,
-                                       omit=NULL,omit.col=NA,...){
+                                       exterr=TRUE,ranked=FALSE,
+                                       hide=NULL, omit=NULL,
+                                       omit.col=NA,...){
     tt <- fissiontrack.age(x,exterr=FALSE)
     # calculated weighted mean age ignoring zeta and rhoD uncertainties
     fit <- weightedmean.default(tt,detect.outliers=detect.outliers,
@@ -451,6 +469,7 @@ weightedmean.fissiontracks <- function(x,random.effects=TRUE,
     }
     if (plot){
         plot_weightedmean(tt[,1],tt[,2],fit=out,from=from,to=to,
+                          levels=levels,clabel=clabel,
                           rect.col=rect.col,outlier.col=outlier.col,
                           sigdig=sigdig,alpha=alpha,units='Ma',
                           ranked=ranked,hide=hide,omit=omit,
@@ -460,11 +479,10 @@ weightedmean.fissiontracks <- function(x,random.effects=TRUE,
 }
 weightedmean_helper <- function(x,random.effects=TRUE,
                                 detect.outliers=TRUE,plot=TRUE,
-                                from=NA,to=NA,
-                                rect.col=grDevices::rgb(0,1,0,0.5),
-                                type=4,cutoff.76=1100,
-                                cutoff.disc=c(-15,5),
-                                outlier.col=grDevices::rgb(0,1,1,0.5),
+                                from=NA,to=NA,levels=NA,clabel="",
+                                rect.col=c("#00FF0080","#FF000080"),
+                                outlier.col="#00FFFF80",type=4,
+                                cutoff.76=1100,cutoff.disc=c(-15,5),
                                 sigdig=2,alpha=0.05,exterr=TRUE,
                                 ranked=FALSE,i2i=FALSE,common.Pb=1,
                                 units='',detritus=0,Th02=c(0,0),
@@ -482,10 +500,16 @@ weightedmean_helper <- function(x,random.effects=TRUE,
         out$mean[c('x','s[x]')] <-
             add.exterr(x,tt=fit$mean['x'],st=fit$mean['s[x]'],
                        cutoff.76=cutoff.76,type=type)
-        out$mean['ci[x]'] <- nfact(alpha)*out$mean['s[x]']
+        if (random.effects){
+            out$mean['ci[x]'] <- nfact(alpha)*out$mean['s[x]']
+        } else {
+            out$mean['ci[x]'] <- tfact(alpha,out$df)*out$mean['s[x]']
+            out$mean['disp[x]'] <- sqrt(out$mswd)*out$mean['ci[x]']
+        }
     }
     if (plot){
         plot_weightedmean(tt[,1],tt[,2],from=from,to=to,fit=out,
+                          levels=levels,clabel=clabel,
                           rect.col=rect.col,outlier.col=outlier.col,
                           sigdig=sigdig,alpha=alpha,units=units,
                           ranked=ranked,hide=hide,omit=omit,
@@ -580,7 +604,7 @@ wtdmean.title <- function(fit,sigdig=2,units='',...){
         rounded.mean <- roundit(fit$mean['x'],
                                 fit$mean[c('s[x]','ci[x]','disp[x]')],
                                 sigdig=sigdig)
-        line1 <- substitute('mean ='~a%+-%b~'|'~c~'|'~d~u~'(n='~n/N~')',
+        line1 <- substitute('mean ='~a%+-%b~'|'~c~'|'~d~u~'(n='*n/N*')',
                             list(a=rounded.mean['x'],
                                  b=rounded.mean['s[x]'],
                                  c=rounded.mean['ci[x]'],
@@ -599,28 +623,31 @@ wtdmean.title <- function(fit,sigdig=2,units='',...){
     mymtext(line2,line=line2line,...)
 }
 
-plot_weightedmean <- function(X,sX,fit,from=NA,to=NA,
-                              rect.col=grDevices::rgb(0,1,0,0.5),
-                              outlier.col=grDevices::rgb(0,1,1,0.5),
-                              sigdig=2,alpha=0.05,units='',
-                              ranked=FALSE,hide=NULL,omit=NULL,
-                              omit.col=NA,...){
-    plotit <- (1:length(X))%ni%hide
-    X <- X[plotit]
-    sX <- sX[plotit]
-    omit <- omit[plotit]
-    if (ranked){
-        i <- order(X)
-        x <- X[i]
-        sx <- sX[i]
-        valid <- fit$valid[i]
-        omit <- omit[i]
-    } else {
-        x <- X
-        sx <- sX
-        valid <- fit$valid
-    }
+plot_weightedmean <- function(X,sX,fit,from=NA,to=NA,levels=NA,clabel="",
+                              rect.col=c("#00FF0080","#FF000080"),
+                              outlier.col="#00FFFF80",sigdig=2,
+                              alpha=0.05,units='',ranked=FALSE,
+                              hide=NULL,omit=NULL,omit.col=NA,...){
+    NS <- length(X)
+    plotit <- (1:NS)%ni%hide
+    calcit <- (1:NS)%ni%c(hide,omit)
+    colour <- set.ellipse.colours(ns=NS,levels=levels,col=rect.col,
+                                  hide=hide,omit=which(!fit$valid),
+                                  omit.col=omit.col)
+    x <- X[plotit]
+    sx <- sX[plotit]
+    valid <- fit$valid[plotit]
+    calcit <- calcit[plotit]
+    colour <- colour[plotit]
     ns <- length(x)
+    if (ranked){
+        i <- order(x)
+        x <- x[i]
+        sx <- sx[i]
+        valid <- valid[i]
+        calcit <- calcit[i]
+        colour <- colour[i]
+    }
     fact <- nfact(alpha)
     if (is.na(from))
         minx <- min(c(x-fact*sx,x-fact*fit$disp['w']),na.rm=TRUE)
@@ -641,12 +668,17 @@ plot_weightedmean <- function(X,sX,fit,from=NA,to=NA,
     graphics::axis(side=1,at=1:ns)
     graphics::axis(side=2)
     for (i in 1:ns){
-        if (i%in%omit) col <- omit.col
-        else if (valid[i]) col <- rect.col
-        else col <- outlier.col
+        if (!calcit[i]){
+            col <- omit.col
+        } else if (valid[i]){
+            col <- colour[i]
+        } else {
+            col <- outlier.col
+        }
         graphics::rect(xleft=i-0.4,ybottom=x[i]-fact*sx[i],
                        xright=i+0.4,ytop=x[i]+fact*sx[i],col=col)
     }
+    colourbar(z=levels[valid],col=rect.col,clabel=clabel)
     graphics::title(wtdmean.title(fit,sigdig=sigdig,units=units))
 }
 
